//Console input handler unit
//  coninp_h.pas
//  (c) by Dust Signs Andreas Unterweger 2006

unit coninp_h;

interface

type
  TStringArray = Array of String;

  THandlerResult = record
    ErrorCode: Integer;
    _Result: String;
    AdditionalData: PChar;
    DataFormat: Integer; //Flag to determine which type of data the AdditionalData-pointer points to
    end;

const
  HR_SUCCESSFUL = 0; //Successful
  HR_PARAMS_INVALID = -1; //Invalid parameters
  HR_NO_HANDLER_FOUND = -2; //No command handler found
  HR_TOO_MANY_PARAMS = -3; //Too many parameters
  HR_TOO_LESS_PARAMS = -4; //Too less parameters
  HR_REQUIRES_EXIT = -5; //Requires exit
  HR_REQUIRES_STATUS_CHANGE = -6; //Requires status change

  DF_NO_DATA = 0; //No data
  DF_STATUS_INFORMATION = 1; //Status information

procedure InitializeHandlerResult(var AHandlerResult: THandlerResult);
procedure SetSpecialHandlerResult(var AHandlerResult: THandlerResult; AResult: Integer);

type
  TCommandHandlerValidatorFunction = function(AParameterList: TStringArray): THandlerResult;
  TCommandHandlerExecutorFunction = function(AParameterList: TStringArray): THandlerResult;

  TCommandHandler = class
    protected
      FHandledCommand: String;
      FValidator: TCommandHandlerValidatorFunction;
      FExecutor: TCommandHandlerExecutorFunction;
    public
      constructor Create(AHandledCommand: String; AValidator: TCommandHandlerValidatorFunction; AnExecutor: TCommandHandlerExecutorFunction);
      function GetHandledCommand: String;
      function Execute(AParameterList: TStringArray): THandlerResult;
    end;

  TConsoleInputHandler = class
    protected
      FCommandHandlers: Array of TCommandHandler;
      function FFindCommandHandler(ACommand: String): TCommandHandler;
    public
      procedure RegisterCommandHandler(ACommandHandler: TCommandHandler);
      function UnregisterCommandHandler(ACommandHandler: TCommandHandler): Boolean;
      function Execute(ACommand: String; AParameterList: TStringArray): THandlerResult;
    end;

implementation

procedure InitializeHandlerResult(var AHandlerResult: THandlerResult);
begin
  with AHandlerResult do begin
    ErrorCode := HR_SUCCESSFUL;
    _Result := '';
    AdditionalData := nil;
    DataFormat := DF_NO_DATA;
    end;
end;

procedure SetSpecialHandlerResult(var AHandlerResult: THandlerResult; AResult: Integer);
begin
  with AHandlerResult do begin
    ErrorCode := AResult;
    case AResult of
      HR_PARAMS_INVALID: _Result := 'Invalid parameters';
      HR_NO_HANDLER_FOUND: _Result := 'No command handler found';
      HR_TOO_MANY_PARAMS: _Result := 'Too many parameters';
      HR_TOO_LESS_PARAMS: _Result := 'Too less parameters';
      end;
    end;
end;

constructor TCommandHandler.Create(AHandledCommand: String; AValidator: TCommandHandlerValidatorFunction; AnExecutor: TCommandHandlerExecutorFunction);
begin
  FHandledCommand := AHandledCommand;
  FValidator := AValidator;
  FExecutor := AnExecutor;
end;

function TCommandHandler.GetHandledCommand: String;
begin
  Result := FHandledCommand;
end;

function TCommandHandler.Execute(AParameterList: TStringArray): THandlerResult;
begin
  InitializeHandlerResult(Result);
  if Assigned(FValidator) then
    Result := FValidator(AParameterList);
  if Result.ErrorCode = HR_SUCCESSFUL then
    Result := FExecutor(AParameterList); //else Result will be the result of Validator
end;

function TConsoleInputHandler.FFindCommandHandler(ACommand: String): TCommandHandler;
var
  i: Integer;
begin
  Result := nil; //No handler found
  for i := 0 to High(FCommandHandlers) do begin
    if FCommandHandlers[i].GetHandledCommand = ACommand then begin
      Result := FCommandHandlers[i];
      break;
      end;
    end;
end;

procedure TConsoleInputHandler.RegisterCommandHandler(ACommandHandler: TCommandHandler);
begin
  SetLength(FCommandHandlers, Length(FCommandHandlers) + 1);
  FCommandHandlers[High(FCommandHandlers)] := ACommandHandler;
end;

function TConsoleInputHandler.UnregisterCommandHandler(ACommandHandler: TCommandHandler): Boolean;
var
  i, j: Integer;
begin
  Result := false; //Not found
  for i := 0 to High(FCommandHandlers) do begin
    if FCommandHandlers[i] = ACommandHandler then begin
      for j := i to High(FCommandHandlers) - 1 do
        FCommandHandlers[j] := FCommandHandlers[j + 1];
      Result := true; //Found and deleted
      exit;
      end;
    end;
end;

function TConsoleInputHandler.Execute(ACommand: String; AParameterList: TStringArray): THandlerResult;
var
  handler: TCommandHandler;
begin
  InitializeHandlerResult(Result);
  handler := FFindCommandHandler(ACommand);
  if handler = nil then begin
    SetSpecialHandlerResult(Result, HR_NO_HANDLER_FOUND);
    exit;
    end;
  Result := handler.Execute(AParameterList);
end;

end.
